@@ -140,8 +140,8 @@ class NelsonAalenFitter(BaseFitter):
         hazard_name = "smoothed-" + cumulative_hazard_name
         hazard_ = self.cumulative_hazard_.diff().fillna(self.cumulative_hazard_.iloc[0])
         C = (hazard_[cumulative_hazard_name] != 0.0).values
-        return pd.DataFrame( 1./(2*bandwidth)*np.dot(epanechnikov_kernel(timeline[:, None], timeline[C][None, :], bandwidth), hazard_.values[C,:]),
-                             columns=[hazard_name], index=timeline)
+        return pd.DataFrame(1. / (2 * bandwidth) * np.dot(epanechnikov_kernel(timeline[:, None], timeline[C][None,:], bandwidth), hazard_.values[C,:]),
+                            columns=[hazard_name], index=timeline)
 
     def smoothed_hazard_confidence_intervals_(self, bandwidth, hazard_=None):
         """
@@ -157,7 +157,7 @@ class NelsonAalenFitter(BaseFitter):
         self._cumulative_sq.iloc[0] = 0
         var_hazard_ = self._cumulative_sq.diff().fillna(self._cumulative_sq.iloc[0])
         C = (var_hazard_.values != 0.0)  # only consider the points with jumps
-        std_hazard_ = np.sqrt(1./(2*bandwidth**2)*np.dot(epanechnikov_kernel(timeline[:, None], timeline[C][None, :], bandwidth)**2, var_hazard_.values[C]))
+        std_hazard_ = np.sqrt(1. / (2 * bandwidth ** 2) * np.dot(epanechnikov_kernel(timeline[:, None], timeline[C][None,:], bandwidth) ** 2, var_hazard_.values[C]))
         values = {
             self.ci_labels[0]: hazard_ * np.exp(alpha2 * std_hazard_ / hazard_),
             self.ci_labels[1]: hazard_ * np.exp(-alpha2 * std_hazard_ / hazard_)
@@ -204,7 +204,6 @@ class KaplanMeierFitter(BaseFitter):
         """
         # if the user is interested in left-censorship, we return the cumulative_density_, no survival_function_,
         estimate_name = 'survival_function_' if not left_censorship else 'cumulative_density_'
-
         v = preprocess_inputs(durations, event_observed, timeline, entry)
         self.durations, self.event_observed, self.timeline, self.entry, self.event_table = v
         self._label = label
@@ -423,7 +422,7 @@ class AalenAdditiveFitter(BaseFitter):
         self.penalizer = penalizer
         assert penalizer >= 0, "penalizer must be >= 0."
 
-    def fit(self, dataframe, duration_col="T", event_col="E",
+    def fit(self, dataframe, duration_col, event_col=None,
             timeline=None, id_col=None, show_progress=True):
         """
         Perform inference on the coefficients of the Aalen additive model.
@@ -456,7 +455,8 @@ class AalenAdditiveFitter(BaseFitter):
                         +----+---+---+------+------+
 
             duration_col: specify what the duration column is called in the dataframe
-            event_col: specify what the event occurred column is called in the dataframe
+            event_col: specify what the event column is called in the dataframe.
+                       If left as None, treat all individuals as non-censored.
             timeline: reformat the estimates index to a new timeline.
             id_col: (only for time-varying covariates) name of the id column in the dataframe
             progress_bar: include a fancy progress bar =)
@@ -475,7 +475,7 @@ class AalenAdditiveFitter(BaseFitter):
 
         return self
 
-    def _fit_static(self, dataframe, duration_col="T", event_col="E",
+    def _fit_static(self, dataframe, duration_col, event_col=None,
                     timeline=None, show_progress=True):
         """
         Perform inference on the coefficients of the Aalen additive model.
@@ -507,8 +507,15 @@ class AalenAdditiveFitter(BaseFitter):
         if self.fit_intercept:
             df['baseline'] = 1.
 
+        # if no event_col is specified, assume all non-censorships
+        if event_col:
+            c = df[event_col].values
+            del df[event_col]
+        else:
+            c = np.ones_like(ids)
+
         # each individual should have an ID of time of leaving study
-        C = pd.Series(df[event_col].values, dtype=bool, index=ids)
+        C = pd.Series(c, dtype=bool, index=ids)
         T = pd.Series(df[duration_col].values, index=ids)
 
         df = df.set_index(id_col)
@@ -516,7 +523,6 @@ class AalenAdditiveFitter(BaseFitter):
         ix = T.argsort()
         T, C = T.iloc[ix], C.iloc[ix]
 
-        del df[event_col]
         del df[duration_col]
         n, d = df.shape
         columns = df.columns
@@ -611,6 +617,11 @@ class AalenAdditiveFitter(BaseFitter):
         # each individual should have an ID of time of leaving study
         df = df.set_index([duration_col, id_col])
 
+        # if no event_col is specified, assume all non-censorships
+        if event_col is None:
+            event_col = 'E'
+            df[event_col] = 1
+
         C_panel = df[[event_col]].to_panel().transpose(2, 1, 0)
         C = C_panel.minor_xs(event_col).sum().astype(bool)
         T = (C_panel.minor_xs(event_col).notnull()).cumsum().idxmax()
@@ -800,7 +811,9 @@ class CoxPHFitter(BaseFitter):
     Parameters:
       alpha: the level in the confidence intervals.
       tie_method: specify how the fitter should deal with ties. Currently only
-         'Efron' is available.
+        'Efron' is available.
+      normalize: substract the mean and divide by standard deviation of each covariate 
+        in the input data before performing any fitting. 
     """
 
     def __init__(self, alpha=0.95, tie_method='Efron', normalize=True):
@@ -1000,7 +1013,7 @@ class CoxPHFitter(BaseFitter):
             print("Convergence completed after %d iterations." % (i))
         return beta
 
-    def fit(self, df, duration_col='T', event_col='E',
+    def fit(self, df, duration_col, event_col=None,
             show_progress=False, initial_beta=None, include_likelihood=False):
         """
         Fit the Cox Propertional Hazard model to a dataset. Tied survival times
@@ -1014,7 +1027,7 @@ class CoxPHFitter(BaseFitter):
           duration_col: the column in dataframe that contains the subjects'
              lifetimes.
           event_col: the column in dataframe that contains the subjects' death
-             observation.
+             observation. If left as None, assume all individuals are non-censored.
           show_progress: since the fitter is iterative, show convergence
              diagnostics.
           initial_beta: initialize the starting point of the iterative
@@ -1030,11 +1043,15 @@ class CoxPHFitter(BaseFitter):
         df = df.copy()
         # Sort on time
         df.sort(duration_col, inplace=True)
+
         # Extract time and event
         T = df[duration_col]
-        E = df[event_col]
         del df[duration_col]
-        del df[event_col]
+        if event_col is None:
+            E = pd.Series(np.ones(df.shape[0]), index=df.index)
+        else:
+            E = df[event_col]
+            del df[event_col]
 
         # Store original non-normalized data
         self.data = df
@@ -1082,7 +1099,7 @@ class CoxPHFitter(BaseFitter):
 
     def _compute_standard_errors(self):
         se = np.sqrt(inv(-self._hessian_).diagonal())
-        return pd.DataFrame(se[None, :],
+        return pd.DataFrame(se[None,:],
                             index=['se'], columns=self.hazards_.columns)
 
     def _compute_z_values(self):
@@ -1093,15 +1110,38 @@ class CoxPHFitter(BaseFitter):
         U = self._compute_z_values() ** 2
         return stats.chi2.sf(U, 1)
 
+    @property
     def summary(self):
+        """Summary statistics describing the fit.
+        Set alpha property in the object before calling.
+
+        Returns
+        -------
+        df : pd.DataFrame
+            Contains columns coef, exp(coef), se(coef), z, p, lower, upper"""
+
         df = pd.DataFrame(index=self.hazards_.columns)
         df['coef'] = self.hazards_.ix['coef'].values
         df['exp(coef)'] = exp(self.hazards_.ix['coef'].values)
         df['se(coef)'] = self._compute_standard_errors().ix['se'].values
         df['z'] = self._compute_z_values()
         df['p'] = self._compute_p_values()
-        df['lower %.2f' % self.alpha] = self.confidence_intervals_.ix['lower-bound'].values
-        df['upper %.2f' % self.alpha] = self.confidence_intervals_.ix['upper-bound'].values
+        df['lower'] = self.confidence_intervals_.ix['lower-bound'].values
+        df['upper'] = self.confidence_intervals_.ix['upper-bound'].values
+        return df
+
+    def print_summary(self, c_index=True):
+        """
+        Print summary statistics describing the fit.
+
+        c_index: If set to False, will not print the concordance index
+
+        """
+        df = self.summary
+        mapper = {'lower': 'lower %.2f' % self.alpha,
+                  'upper': 'upper %.2f' % self.alpha}
+        df = df.rename_axis(mapper, axis=1)
+
         # Significance codes last
         df[''] = [significance_code(p) for p in df['p']]
 
@@ -1114,10 +1154,11 @@ class CoxPHFitter(BaseFitter):
         print('---')
         print("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1 ",
               end='\n\n')
-        print("Concordance = {:.3f}"
-              .format(concordance_index(self.durations,
-                                        -self.predict_partial_hazard(self.data).values.ravel(),
-                                        self.event_observed)))
+        if c_index:
+            print("Concordance = {:.3f}"
+                  .format(concordance_index(self.durations,
+                                            -self.predict_partial_hazard(self.data).values.ravel(),
+                                            self.event_observed)))
         return
 
     def predict_partial_hazard(self, X):
@@ -1197,8 +1238,7 @@ class CoxPHFitter(BaseFitter):
         ind_hazards = self.predict_partial_hazard(self.data).values
 
         event_table = survival_table_from_events(self.durations.values,
-                                                 self.event_observed.values,
-                                                 np.zeros_like(self.durations))
+                                                 self.event_observed.values)
 
         baseline_hazard_ = pd.DataFrame(np.zeros((event_table.shape[0], 1)),
                                         index=event_table.index,
@@ -1215,7 +1255,6 @@ class CoxPHFitter(BaseFitter):
         return baseline_hazard_
 
 
-#### Utils ####
 def get_index(X):
     if isinstance(X, pd.DataFrame):
         index = list(X.index)
@@ -1238,8 +1277,10 @@ def _subtract(self, estimate):
     def subtract(other):
         self_estimate = getattr(self, estimate)
         other_estimate = getattr(other, estimate)
-        return self_estimate.reindex(other_estimate.index, method='ffill') - \
-            other_estimate.reindex(self_estimate.index, method='ffill')
+        new_index = np.concatenate((other_estimate.index, self_estimate.index))
+        new_index = np.unique(new_index)
+        return self_estimate.reindex(new_index, method='ffill') - \
+            other_estimate.reindex(new_index, method='ffill')
 
     subtract.__doc__ = doc_string
     return subtract
@@ -1258,15 +1299,17 @@ def _divide(self, estimate):
     def divide(other):
         self_estimate = getattr(self, estimate)
         other_estimate = getattr(other, estimate)
-        return self_estimate.reindex(other_estimate.index, method='ffill') / \
-            other_estimate.reindex(self_estimate.index, method='ffill')
+        new_index = np.concatenate((other_estimate.index, self_estimate.index))
+        new_index = np.unique(new_index)
+        return self_estimate.reindex(new_index, method='ffill') / \
+            other_estimate.reindex(new_index, method='ffill')
 
     divide.__doc__ = doc_string
     return divide
 
 
 def _predict(self, estimate, label):
-    doc_string =       """
+    doc_string = """
       Predict the %s at certain times
 
       Parameters:
@@ -1291,13 +1334,10 @@ def preprocess_inputs(durations, event_observed, timeline, entry):
     else:
         event_observed = np.asarray(event_observed).reshape((n,)).copy().astype(int)
 
-    if entry is None:
-        entry = np.zeros(n)
-    else:
+    if entry is not None:
         entry = np.asarray(entry).reshape((n,))
 
     event_table = survival_table_from_events(durations, event_observed, entry)
-
     if timeline is None:
         timeline = event_table.index.values
     else:
@@ -1353,7 +1393,7 @@ def qth_survival_times(q, survival_functions):
     if survival_functions.shape[1] == 1 and q.shape == (1,):
         return survival_functions.apply(lambda s: qth_survival_time(q[0], s)).ix[0]
     else:
-        return pd.DataFrame({_q: survival_functions.apply(lambda s: qth_survival_time(_q, s)) for _q in q})
+        return pd.DataFrame({q_: survival_functions.apply(lambda s: qth_survival_time(q_, s)) for q_ in q})
 
 
 def qth_survival_time(q, survival_function):
