@@ -3,12 +3,173 @@ from __future__ import print_function
 
 # plotting
 import numpy as np
+import matplotlib as mpl
 from matplotlib import pyplot as plt
 
 from lifelines.utils import coalesce
 
 
-def plot_lifetimes(lifetimes, event_observed=None, birthtimes=None, order=False):
+def is_latex_enabled():
+    '''
+    Returns True if LaTeX is enabled in matplotlib's rcParams,
+    False otherwise
+    '''
+    return mpl.rcParams['text.usetex']
+
+
+def remove_spines(ax, sides):
+    '''
+    Remove spines of axis.
+
+    Parameters:
+      ax: axes to operate on
+      sides: list of sides: top, left, bottom, right
+
+    Examples:
+    removespines(ax, ['top'])
+    removespines(ax, ['top', 'bottom', 'right', 'left'])
+    '''
+    for side in sides:
+        ax.spines[side].set_visible(False)
+    return ax
+
+
+def move_spines(ax, sides, dists):
+    '''
+    Move the entire spine relative to the figure.
+
+    Parameters:
+      ax: axes to operate on
+      sides: list of sides to move. Sides: top, left, bottom, right
+      dists: list of float distances to move. Should match sides in length.
+
+    Example:
+    move_spines(ax, sides=['left', 'bottom'], dists=[-0.02, 0.1])
+    '''
+    for side, dist in zip(sides, dists):
+        ax.spines[side].set_position(('axes', dist))
+    return ax
+
+
+def remove_ticks(ax, x=False, y=False):
+    '''
+    Remove ticks from axis.
+
+    Parameters:
+      ax: axes to work on
+      x: if True, remove xticks. Default False.
+      y: if True, remove yticks. Default False.
+
+    Examples:
+    removeticks(ax, x=True)
+    removeticks(ax, x=True, y=True)
+    '''
+    if x:
+        ax.xaxis.set_ticks_position('none')
+    if y:
+        ax.yaxis.set_ticks_position('none')
+    return ax
+
+
+def add_at_risk_counts(*fitters, **kwargs):
+    '''
+    Add counts showing how many individuals were at risk at each time point in
+    survival/hazard plots.
+
+    Arguments:
+      One or several fitters, for example KaplanMeierFitter,
+      NelsonAalenFitter, etc...
+
+    Keyword arguments (all optional):
+      ax: The axes to add the labels to. Default is the current axes.
+      fig: The figure of the axes. Default is the current figure.
+      labels: The labels to use for the fitters. Default is whatever was
+              specified in the fitters' fit-function. Giving 'None' will
+              hide fitter labels.
+
+    Returns:
+      ax: The axes which was used.
+
+    Examples:
+        # First train some fitters and plot them
+        fig = plt.figure()
+        ax = plt.subplot(111)
+
+        f1 = KaplanMeierFitter()
+        f1.fit(data)
+        f1.plot(ax=ax)
+
+        f2 = KaplanMeierFitter()
+        f2.fit(data)
+        f2.plot(ax=ax)
+
+        # There are equivalent
+        add_at_risk_counts(f1, f2)
+        add_at_risk_counts(f1, f2, ax=ax, fig=fig)
+        # This overrides the labels
+        add_at_risk_counts(f1, f2, labels=['fitter one', 'fitter two'])
+        # This hides the labels
+        add_at_risk_counts(f1, f2, labels=None)
+    '''
+    # Axes and Figure can't be None
+    ax = kwargs.get('ax', None)
+    if ax is None:
+        ax = plt.gca()
+
+    fig = kwargs.get('fig', None)
+    if fig is None:
+        fig = plt.gcf()
+
+    if 'labels' not in kwargs:
+        labels = [f._label for f in fitters]
+    else:
+        # Allow None, in which case no labels should be used
+        labels = kwargs['labels']
+        if labels is None:
+            labels = [None] * len(fitters)
+    # Create another axes where we can put size ticks
+    ax2 = plt.twiny(ax=ax)
+    # Move the ticks below existing axes
+    # Appropriate length scaled for 6 inches. Adjust for figure size.
+    ax2_ypos = -0.15 * 6.0 / fig.get_figheight()
+    move_spines(ax2, ['bottom'], [ax2_ypos])
+    # Hide all fluff
+    remove_spines(ax2, ['top', 'right', 'bottom', 'left'])
+    # Set ticks and labels on bottom
+    ax2.xaxis.tick_bottom()
+    # Match tick numbers and locations
+    ax2.set_xlim(ax.get_xlim())
+    ax2.set_xticks(ax.get_xticks())
+    # Remove ticks, need to do this AFTER moving the ticks
+    remove_ticks(ax2, x=True, y=True)
+    # Add population size at times
+    ticklabels = []
+    for tick in ax2.get_xticks():
+        lbl = ""
+        for f, l in zip(fitters, labels):
+            # First tick is prepended with the label
+            if tick == ax2.get_xticks()[0] and l is not None:
+                if is_latex_enabled():
+                    s = "\n{}\\quad".format(l) + "{}"
+                else:
+                    s = "\n{}   ".format(l) + "{}"
+            else:
+                s = "\n{}"
+            lbl += s.format(f.durations[f.durations >= tick].shape[0])
+        ticklabels.append(lbl.strip())
+    # Align labels to the right so numbers can be compared easily
+    ax2.set_xticklabels(ticklabels, ha='right')
+
+    # Add a descriptive headline.
+    ax2.xaxis.set_label_coords(0, ax2_ypos)
+    ax2.set_xlabel('At risk')
+
+    plt.tight_layout()
+    return ax
+
+
+def plot_lifetimes(lifetimes, event_observed=None, birthtimes=None,
+                   order=False):
     """
     Parameters:
       lifetimes: an (n,) numpy array of lifetimes.
@@ -51,7 +212,8 @@ def plot_lifetimes(lifetimes, event_observed=None, birthtimes=None, order=False)
 def shaded_plot(x, y, y_upper, y_lower, **kwargs):
     ax = kwargs.pop('ax', plt.gca())
     base_line, = ax.plot(x, y, drawstyle='steps-post', **kwargs)
-    fill_between_steps(x, y_lower, y2=y_upper, ax=ax, alpha=0.25, color=base_line.get_color(), linewidth=1.0)
+    fill_between_steps(x, y_lower, y2=y_upper, ax=ax, alpha=0.25,
+                       color=base_line.get_color(), linewidth=1.0)
     return
 
 
@@ -104,33 +266,48 @@ def plot_estimate(self, estimate):
         Matplotlib plot arguments can be passed in inside the kwargs, plus
 
         Parameters:
-          flat: an opiniated design style with stepped lines and no shading. Similar to R's plotting. Default: False
+          flat: an opiniated design style with stepped lines and no shading.
+                Similar to R's plotting. Default: False
           show_censors: place markers at censorship events. Default: False
-          censor_styles: If show_censors, this dictionary will be passed into the plot call.
-          ci_alpha: the transparency level of the confidence interval. Default: 0.3
-          ci_force_lines: force the confidence intervals to be line plots (versus default shaded areas). Default: False
-          ci_show=True: show confidence intervals. Default: True
-          ci_legend: if ci_force_lines is True, this is a boolean flag to add the lines' labels to the legend. Default: False
+          censor_styles: If show_censors, this dictionary will be passed into
+                         the plot call.
+          ci_alpha: the transparency level of the confidence interval.
+                    Default: 0.3
+          ci_force_lines: force the confidence intervals to be line plots
+                          (versus default shaded areas). Default: False
+          ci_show: show confidence intervals. Default: True
+          ci_legend: if ci_force_lines is True, this is a boolean flag to add
+                     the lines' labels to the legend. Default: False
+          at_risk_counts: show group sizes at time points. See function
+                          'add_at_risk_counts' for details. Default: False
           ix: specify a time-based subsection of the curves to plot, ex:
-                   .plot(ix=slice(0.,10.)) will plot the time values between t=0. and t=10.
+                   .plot(ix=slice(0.,10.))
+              will plot the time values between t=0. and t=10.
           iloc: specify a location-based subsection of the curves to plot, ex:
-                   .plot(iloc=slice(0,10)) will plot the first 10 time points.
-          bandwidth: specify the bandwidth of the kernel smoother for the smoothed-hazard rate. Only used
-              when called 'plot_hazard'.
+                   .plot(iloc=slice(0,10))
+                will plot the first 10 time points.
+          bandwidth: specify the bandwidth of the kernel smoother for the
+                     smoothed-hazard rate. Only used when called 'plot_hazard'.
 
         Returns:
           ax: a pyplot axis object
         """ % estimate
 
-    def plot(ix=None, iloc=None, flat=False, show_censors=False, censor_styles={},
-             ci_legend=False, ci_force_lines=False, ci_alpha=0.25, ci_show=True,
+    def plot(ix=None, iloc=None, flat=False, show_censors=False,
+             censor_styles=None, ci_legend=False, ci_force_lines=False,
+             ci_alpha=0.25, ci_show=True, at_risk_counts=False,
              bandwidth=None, **kwargs):
 
-        assert (ix is None or iloc is None), 'Cannot set both ix and iloc in call to .plot().'
+        if censor_styles is None:
+            censor_styles = {}
+
+        if (ix is not None and iloc is not None):
+            raise ValueError('Cannot set both ix and iloc in call to .plot().')
 
         if "ax" not in kwargs:
             kwargs["ax"] = plt.figure().add_subplot(111)
-        kwargs['color'] = coalesce(kwargs.get('c'), kwargs.get('color'), next(kwargs["ax"]._get_lines.color_cycle))
+        kwargs['color'] = coalesce(kwargs.get('c'), kwargs.get('color'),
+                                   next(kwargs["ax"]._get_lines.color_cycle))
         kwargs['drawstyle'] = coalesce(kwargs.get('drawstyle'), 'steps-post')
 
         # R-style graphics
@@ -139,16 +316,20 @@ def plot_estimate(self, estimate):
             show_censors = True
 
         if estimate == "hazard_":
-            assert bandwidth is not None, 'Must specify a bandwidth parameter in the call to plot_hazard.'
+            if bandwidth is None:
+                raise ValueError('Must specify a bandwidth parameter in the ' +
+                                 'call to plot_hazard.')
             estimate_ = self.smoothed_hazard_(bandwidth)
-            confidence_interval_ = self.smoothed_hazard_confidence_intervals_(bandwidth, hazard_=estimate_.values[:, 0])
+            confidence_interval_ = \
+                self.smoothed_hazard_confidence_intervals_(bandwidth,
+                                                           hazard_=estimate_.values[:, 0])
         else:
             confidence_interval_ = getattr(self, 'confidence_interval_')
             estimate_ = getattr(self, estimate)
 
         # did user specify certain indexes or locations?
         if iloc == ix is None:
-            user_submitted_ix = slice(0, None)
+            user_submitted_ix = slice(None)
         else:
             user_submitted_ix = ix if ix is not None else iloc
 
@@ -161,9 +342,10 @@ def plot_estimate(self, estimate):
             cs.update(censor_styles)
             times = get_loc(self.event_table.ix[(self.event_table['censored'] > 0)]).index.values.astype(float)
             v = self.predict(times)
-            kwargs['ax'].plot(times, v, linestyle='None', color=kwargs['color'], **cs)
+            kwargs['ax'].plot(times, v, linestyle='None',
+                              color=kwargs['color'], **cs)
 
-        # plot esimate
+        # plot estimate
         get_loc(estimate_).plot(**kwargs)
 
         # plot confidence intervals
@@ -177,7 +359,12 @@ def plot_estimate(self, estimate):
                 x = get_loc(confidence_interval_).index.values.astype(float)
                 lower = get_loc(confidence_interval_.filter(like='lower')).values[:, 0]
                 upper = get_loc(confidence_interval_.filter(like='upper')).values[:, 0]
-                fill_between_steps(x, lower, y2=upper, ax=kwargs['ax'], alpha=ci_alpha, color=kwargs['color'], linewidth=1.0)
+                fill_between_steps(x, lower, y2=upper, ax=kwargs['ax'],
+                                   alpha=ci_alpha, color=kwargs['color'],
+                                   linewidth=1.0)
+
+        if at_risk_counts:
+            add_at_risk_counts(self, ax=kwargs['ax'])
 
         return kwargs['ax']
     plot.__doc__ = doc_string
