@@ -7,13 +7,13 @@ import pandas as pd
 import pytest
 from matplotlib import pyplot as plt
 
-from pandas.util.testing import assert_frame_equal
+from pandas.util.testing import assert_frame_equal, assert_series_equal
 import numpy.testing as npt
 
 from ..utils import k_fold_cross_validation, StatError
 from ..estimation import CoxPHFitter, AalenAdditiveFitter, KaplanMeierFitter, NelsonAalenFitter, BreslowFlemingHarringtonFitter
 from ..datasets import load_regression_dataset, load_larynx, load_waltons, load_kidney_transplant, load_rossi,\
-    load_lcd, load_panel_test
+    load_lcd, load_panel_test, load_g3
 from ..generate_datasets import generate_hazard_rates, generate_random_lifetimes, cumulative_integral
 from ..utils import concordance_index
 
@@ -24,6 +24,12 @@ def sample_lifetimes():
     return (np.random.randint(20, size=N), np.random.randint(2, size=N))
 
 
+@pytest.fixture
+def sample_lifetimes2():
+    N = 25
+    return (np.random.randint(20, size=N), np.random.randint(2, size=N))
+
+
 @pytest.fixture
 def waltons_dataset():
     return load_waltons()
@@ -115,15 +121,31 @@ class TestUnivariateFitters():
         with_list = naf.fit(list(T), list(C)).cumulative_hazard_
         assert_frame_equal(with_list, with_array)
 
-    def test_subtraction_function(self, sample_lifetimes):
-        for f in [KaplanMeierFitter(), NelsonAalenFitter()]:
-            f.fit(sample_lifetimes[0])
-            npt.assert_array_almost_equal(f.subtract(f).sum().values, 0.0)
+    def test_subtraction_function(self, sample_lifetimes, sample_lifetimes2):
+        for fitter in [KaplanMeierFitter, NelsonAalenFitter]:
+            f1 = fitter()
+            f2 = fitter()
 
-    def test_divide_function(self, sample_lifetimes):
-        for f in [KaplanMeierFitter(), NelsonAalenFitter()]:
-            f.fit(sample_lifetimes[0])
-            npt.assert_array_almost_equal(np.log(f.divide(f)).sum().values, 0.0)
+            f1.fit(sample_lifetimes[0])
+            f2.fit(sample_lifetimes2[0])
+
+            result = f1.subtract(f2)
+            assert result.shape[0] == (np.unique(np.concatenate((f1.timeline, f2.timeline))).shape[0])
+
+            npt.assert_array_almost_equal(f1.subtract(f1).sum().values, 0.0)
+
+    def test_divide_function(self, sample_lifetimes, sample_lifetimes2):
+        for fitter in [KaplanMeierFitter, NelsonAalenFitter]:
+            f1 = fitter()
+            f2 = fitter()
+
+            f1.fit(sample_lifetimes[0])
+            f2.fit(sample_lifetimes2[0])
+
+            result = f1.subtract(f2)
+            assert result.shape[0] == (np.unique(np.concatenate((f1.timeline, f2.timeline))).shape[0])
+
+            npt.assert_array_almost_equal(np.log(f1.divide(f1)).sum().values, 0.0)
 
 
 class TestKaplanMeierFitter():
@@ -163,10 +185,9 @@ class TestKaplanMeierFitter():
         npt.assert_almost_equal(kmf.survival_function_.values, self.kaplan_meier(T, C))
 
     def test_stat_error_is_raised_if_too_few_early_deaths(self):
-        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30,
-                           41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
-        observations = np.array([1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31,
+        observations = np.array([1,  1,  1, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31,
                                  22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
+        births = observations - 1
         kmf = KaplanMeierFitter()
         with pytest.raises(StatError):
             kmf.fit(observations, entry=births)
@@ -192,6 +213,20 @@ class TestKaplanMeierFitter():
         assert kmf.cumulative_density_[kmf._label].ix[0] == 0.0
         assert kmf.cumulative_density_[kmf._label].ix[12] == 1.0
 
+    def test_shifting_durations_doesnt_affect_survival_function_values(self):
+        T = np.random.exponential(10, size=100)
+        kmf = KaplanMeierFitter()
+        expected = kmf.fit(T).survival_function_.values
+
+        T_shifted = T + 100
+        npt.assert_almost_equal(expected, kmf.fit(T_shifted).survival_function_.values)
+
+        T_shifted = T - 50
+        npt.assert_almost_equal(expected[1:], kmf.fit(T_shifted).survival_function_.values)
+
+        T_shifted = T - 200
+        npt.assert_almost_equal(expected[1:], kmf.fit(T_shifted).survival_function_.values)
+
     @pytest.mark.plottest
     @pytest.mark.skipif("DISPLAY" not in os.environ, reason="requires display")
     def test_kmf_left_censorship_plots(self):
@@ -207,6 +242,34 @@ class TestKaplanMeierFitter():
         plt.show()
         return
 
+    def test_kmf_survival_curve_output_against_R(self):
+        df = load_g3()
+        ix = df['group'] == 'RIT'
+        kmf = KaplanMeierFitter()
+
+        expected = np.array([[0.909, 0.779]]).T
+        kmf.fit(df.ix[ix]['time'], df.ix[ix]['event'], timeline=[25, 53])
+        npt.assert_array_almost_equal(kmf.survival_function_.values, expected, decimal=3)
+
+        expected = np.array([[0.833, 0.667, 0.5, 0.333]]).T
+        kmf.fit(df.ix[~ix]['time'], df.ix[~ix]['event'], timeline=[9, 19, 32, 34])
+        npt.assert_array_almost_equal(kmf.survival_function_.values, expected, decimal=3)
+
+    def test_kmf_confidence_intervals_output_against_R(self):
+        # this uses conf.type = 'log-log'
+        df = load_g3()
+        ix = df['group'] != 'RIT'
+        kmf = KaplanMeierFitter()
+        kmf.fit(df.ix[ix]['time'], df.ix[ix]['event'], timeline=[9, 19, 32, 34])
+
+        expected_lower_bound = np.array([0.2731, 0.1946, 0.1109, 0.0461])
+        npt.assert_array_almost_equal(kmf.confidence_interval_['KM-estimate_lower_0.95'].values,
+                                      expected_lower_bound, decimal=3)
+
+        expected_upper_bound = np.array([0.975, 0.904, 0.804, 0.676])
+        npt.assert_array_almost_equal(kmf.confidence_interval_['KM-estimate_upper_0.95'].values,
+                                      expected_upper_bound, decimal=3)
+
 
 class TestNelsonAalenFitter():
 
@@ -288,16 +351,43 @@ class TestBreslowFlemingHarringtonFitter():
 
     def test_BHF_fit(self):
         bfh = BreslowFlemingHarringtonFitter()
-        births = np.array([51, 58, 55, 28, 21, 19, 25, 48, 47, 25, 31, 24, 25, 30, 33, 36, 30,
-                           41, 43, 45, 35, 29, 35, 32, 36, 32, 10])
+
         observations = np.array([1,  1,  2, 22, 30, 28, 32, 11, 14, 36, 31, 33, 33, 37, 35, 25, 31,
                                  22, 26, 24, 35, 34, 30, 35, 40, 39,  2])
+        births = observations - 1
         bfh.fit(observations, entry=births)
-        return
 
 
 class TestRegressionFitters():
 
+    def test_fit_methods_require_duration_col(self):
+        X = load_regression_dataset()
+
+        aaf = AalenAdditiveFitter()
+        cph = CoxPHFitter()
+
+        with pytest.raises(TypeError):
+            aaf.fit(X)
+        with pytest.raises(TypeError):
+            cph.fit(X)
+
+    def test_fit_methods_can_accept_optional_event_col_param(self):
+        X = load_regression_dataset()
+
+        aaf = AalenAdditiveFitter()
+        aaf.fit(X, 'T', event_col='E')
+        assert_series_equal(aaf.event_observed.sort_index(), X['E'].astype(bool))
+
+        aaf.fit(X, 'T')
+        npt.assert_array_equal(aaf.event_observed.values, np.ones(X.shape[0]))
+
+        cph = CoxPHFitter()
+        cph.fit(X, 'T', event_col='E')
+        assert_series_equal(cph.event_observed.sort_index(), X['E'].astype(bool))
+
+        cph.fit(X, 'T')
+        npt.assert_array_equal(cph.event_observed.values, np.ones(X.shape[0]))
+
     def test_predict_methods_in_regression_return_same_types(self):
         X = load_regression_dataset()
         x = X[X.columns - ['T', 'E']]
@@ -328,14 +418,14 @@ class TestRegressionFitters():
         expected_index = pd.Index(np.array([3, 2, 1, 0]))
 
         cph = CoxPHFitter()
-        cph.fit(data_pred2, duration_col='t')
+        cph.fit(data_pred2, duration_col='t', event_col='E')
         npt.assert_array_equal(cph.predict_partial_hazard(x).index, expected_index)
         npt.assert_array_equal(cph.predict_percentile(x).index, expected_index)
         npt.assert_array_equal(cph.predict(x).index, expected_index)
         npt.assert_array_equal(cph.predict_expectation(x).index, expected_index)
 
         aaf = AalenAdditiveFitter()
-        aaf.fit(data_pred2, duration_col='t')
+        aaf.fit(data_pred2, duration_col='t', event_col='E')
         npt.assert_array_equal(aaf.predict_percentile(x).index, expected_index)
         npt.assert_array_equal(aaf.predict(x).index, expected_index)
         npt.assert_array_equal(aaf.predict_expectation(x).index, expected_index)
@@ -343,6 +433,21 @@ class TestRegressionFitters():
 
 class TestCoxPHFitter():
 
+    def test_summary(self):
+
+        cp = CoxPHFitter()
+        df = load_rossi()
+        cp.fit(df, duration_col='week', event_col='arrest')
+        summDf = cp.summary
+        expectedColumns = ['coef',
+                           'exp(coef)',
+                           'se(coef)',
+                           'z',
+                           'p',
+                           'lower',
+                           'upper']
+        assert all([col in summDf.columns for col in expectedColumns])
+
     def test_print_summary(self):
 
         import sys
@@ -359,7 +464,7 @@ class TestCoxPHFitter():
             cp = CoxPHFitter()
             df = load_rossi()
             cp.fit(df, duration_col='week', event_col='arrest')
-            cp.summary()
+            cp.print_summary()
             output = out.getvalue().strip().split()
             expected = """n=432, number of events=114
 
@@ -375,7 +480,7 @@ prio  2.639e-01  1.302e+00 8.291e-02  3.182e+00 1.460e-03   1.013e-01   4.264e-0
 Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
 
 Concordance = 0.642""".strip().split()
-            for i in [0,1,2,-2,-1]:
+            for i in [0, 1, 2, -2, -1]:
                 assert output[i] == expected[i]
         finally:
             sys.stdout = saved_stdout
@@ -659,9 +764,8 @@ class TestAalenAdditiveFitter():
         X = pd.DataFrame(np.random.randn(n, d))
         T = np.random.exponential(size=n)
         X['T'] = T
-        X['E'] = 1
         aaf = AalenAdditiveFitter()
-        aaf.fit(X)
+        aaf.fit(X, duration_col='T')
 
     @pytest.mark.plottest
     @pytest.mark.skipif("DISPLAY" not in os.environ, reason="requires display")
@@ -671,6 +775,14 @@ class TestAalenAdditiveFitter():
         aaf.fit(panel_dataset, id_col='id', duration_col='t', event_col='E')
         aaf.plot()
 
+    def test_aaf_panel_dataset_with_no_censorship(self):
+        panel_dataset = load_panel_test()
+        aaf = AalenAdditiveFitter()
+        aaf.fit(panel_dataset, id_col='id', duration_col='t')
+        expected = pd.Series([True] * 9, index=range(1, 10))
+        expected.index.name = 'id'
+        assert_series_equal(aaf.event_observed, expected)
+
     def test_aalen_additive_median_predictions_split_data(self):
         # This tests to make sure that my median predictions statisfy
         # the prediction are greater than the actual 1/2 the time.
@@ -681,10 +793,9 @@ class TestAalenAdditiveFitter():
         hz, coef, X = generate_hazard_rates(n, d, timeline)
         T = generate_random_lifetimes(hz, timeline)
         X['T'] = T
-        X['E'] = 1
         # fit it to Aalen's model
         aaf = AalenAdditiveFitter()
-        aaf.fit(X)
+        aaf.fit(X, 'T')
 
         # predictions
         T_pred = aaf.predict_median(X[list(range(6))])
@@ -706,7 +817,7 @@ class TestAalenAdditiveFitter():
         X['T'] = T
         X['E'] = np.random.binomial(1, 1, n)
         aaf = AalenAdditiveFitter()
-        aaf.fit(X)
+        aaf.fit(X, 'T', 'E')
 
         for i in range(d + 1):
             ax = plt.subplot(d + 1, 1, i + 1)
@@ -733,7 +844,7 @@ class TestAalenAdditiveFitter():
         X['E'] = np.random.binomial(1, 0.99, n)
 
         aaf = AalenAdditiveFitter()
-        aaf.fit(X)
+        aaf.fit(X, 'T', 'E')
 
         for i in range(d + 1):
             ax = plt.subplot(d + 1, 1, i + 1)
