@@ -60,6 +60,10 @@ def concordance_index(event_times, predicted_event_times, event_observed=None):
 
     if event_observed is None:
         event_observed = np.ones(event_times.shape[0], dtype=float)
+    else:
+        if event_observed.shape != event_times.shape:
+            raise ValueError("Observed events must be 1-dimensional of same length as event times")
+        event_observed = np.array(event_observed, dtype=float).ravel()
 
     # 100 times faster to calculate in Fortran
     return _cindex(event_times,
@@ -71,7 +75,7 @@ def coalesce(*args):
     return next(s for s in args if s is not None)
 
 
-def group_survival_table_from_events(groups, durations, event_observed, min_observations, limit=-1):
+def group_survival_table_from_events(groups, durations, event_observed, birth_times, limit=-1):
     """
     Joins multiple event series together into dataframes. A generalization of
     `survival_table_from_events` to data with groups. Previously called `group_event_series` pre 0.2.3.
@@ -80,9 +84,9 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
         groups: a (n,) array of individuals' group ids.
         durations: a (n,)  array of durations of each individual
         event_observed: a (n,) array of event observations, 1 if observed, 0 else.
-        min_observations: a (n,) array of times individual entered study. This is most applicable in
-                    cases where there is left-truncation, i.e. a individual might enter the
-                    study late. If not the case, normally set to all zeros.
+        birth_times: a (n,) array of numbers representing
+          when the subject was first observed. A subject's death event is then at [birth times + duration observed].
+          Normally set to all zeros, but can be positive or negative.
 
     Output:
         - np.array of unique groups
@@ -124,8 +128,8 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
 
     """
     n = max(groups.shape)
-    assert n == max(durations.shape) == max(event_observed.shape) == max(min_observations.shape), "inputs must be of the same length."
-    groups, durations, event_observed, min_observations = map(lambda x: pd.Series(np.reshape(x, (n,))), [groups, durations, event_observed, min_observations])
+    assert n == max(durations.shape) == max(event_observed.shape) == max(birth_times.shape), "inputs must be of the same length."
+    groups, durations, event_observed, birth_times = map(lambda x: pd.Series(np.reshape(x, (n,))), [groups, durations, event_observed, birth_times])
     unique_groups = groups.unique()
 
     # set first group
@@ -133,7 +137,7 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
     ix = (groups == g)
     T = durations[ix]
     C = event_observed[ix]
-    B = min_observations[ix]
+    B = birth_times[ix]
 
     g_name = str(g)
     data = survival_table_from_events(T, C, B,
@@ -142,7 +146,7 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
         ix = groups == g
         T = durations[ix]
         C = event_observed[ix]
-        B = min_observations[ix]
+        B = birth_times[ix]
         g_name = str(g)
         data = data.join(survival_table_from_events(T, C, B,
                                                     columns=['removed:' + g_name, "observed:" + g_name, 'censored:' + g_name, 'entrance' + g_name]),
@@ -154,15 +158,15 @@ def group_survival_table_from_events(groups, durations, event_observed, min_obse
     return unique_groups, data.filter(like='removed:'), data.filter(like='observed:'), data.filter(like='censored:')
 
 
-def survival_table_from_events(durations, event_observed, min_observations,
+def survival_table_from_events(death_times, event_observed, birth_times=None,
                                columns=["removed", "observed", "censored", "entrance"], weights=None):
     """
     Parameters:
-        durations: (n,1) array of event times (durations individual was observed for)
-        event_observed: (n,1) boolean array, 1 if observed event, 0 is censored event.
-        min_observations: used for left truncation data. Sometimes subjects will show
-          up late in the study. min_observations is a (n,1) array of positive numbers representing
-          when the subject was first observed. A subject's life is then [min observation + duration observed]
+        death_times: (n,) array of event times
+        event_observed: (n,) boolean array, 1 if observed event, 0 is censored event.
+        birth_times: a (n,) array of numbers representing
+          when the subject was first observed. A subject's death event is then at [birth times + duration observed].
+          If None (default), birth_times are set to be the first observation or 0, which ever is smaller.
         columns: a 3-length array to call the, in order, removed individuals, observed deaths
           and censorships.
         weights: Default None, otherwise (n,1) array. Optional argument to use weights for individuals.
@@ -190,21 +194,26 @@ def survival_table_from_events(durations, event_observed, min_observations,
         15              2         2         0         0
 
     """
-    # deal with deaths and censorships
+    death_times = np.asarray(death_times)
+    if birth_times is None:
+        birth_times = min(0, death_times.min()) * np.ones(death_times.shape[0])
+    else:
+        birth_times = np.asarray(birth_times)
+        if np.any(birth_times > death_times):
+            raise ValueError('birth time must be less than time of death.')
 
-    durations = np.asarray(durations) + min_observations
-    df = pd.DataFrame(durations, columns=["event_at"])
+    # deal with deaths and censorships
+    df = pd.DataFrame(death_times, columns=["event_at"])
     df[columns[0]] = 1 if weights is None else weights
-    df[columns[1]] = event_observed
+    df[columns[1]] = np.asarray(event_observed)
     death_table = df.groupby("event_at").sum()
     death_table[columns[2]] = (death_table[columns[0]] - death_table[columns[1]]).astype(int)
 
     # deal with late births
-    births = pd.DataFrame(min_observations, columns=['event_at'])
+    births = pd.DataFrame(birth_times, columns=['event_at'])
     births[columns[3]] = 1
     births_table = births.groupby('event_at').sum()
 
-    # this next line can be optimized for when min_observerations is all zeros.
     event_table = death_table.join(births_table, how='outer', sort=True).fillna(0)  # http://wesmckinney.com/blog/?p=414
     return event_table.astype(float)
 
@@ -322,7 +331,7 @@ def AandS_approximation(p):
     return t - (c_0 + c_1 * t + c_2 * t ** 2) / (1 + d_1 * t + d_2 * t * t + d_3 * t ** 3)
 
 
-def k_fold_cross_validation(fitter, df, duration_col='T', event_col='E',
+def k_fold_cross_validation(fitter, df, duration_col, event_col=None,
                             k=5, evaluation_measure=concordance_index, predictor="predict_median",
                             predictor_kwargs={}):
     """
@@ -333,7 +342,8 @@ def k_fold_cross_validation(fitter, df, duration_col='T', event_col='E',
         other covariates. `duration_col` refers to the lifetimes of the subjects. `event_col`
         refers to whether the 'death' events was observed: 1 if observed, 0 else (censored).
     duration_col: the column in dataframe that contains the subjects lifetimes.
-    event_col: the column in dataframe that contains the subject's death observation.    loss functions:
+    event_col: the column in dataframe that contains the subject's death observation. If left
+                as None, assumes all individuals are non-censored.   
     k: the number of folds to perform. n/k data will be withheld for testing on.
     evaluation_measure: a function that accepts either (event_times, predicted_event_times),
                   or (event_times, predicted_event_times, event_observed) and returns a scalar value.
@@ -348,6 +358,11 @@ def k_fold_cross_validation(fitter, df, duration_col='T', event_col='E',
     n, d = df.shape
     scores = np.zeros((k,))
     df = df.copy()
+
+    if event_col is None:
+        event_col = 'E'
+        df[event_col] = 1.
+
     df = df.reindex(np.random.permutation(df.index)).sort(event_col)
 
     assignments = np.array((n // k + 1) * list(range(1, k + 1)))
@@ -397,7 +412,7 @@ def unnormalize(X, mean, std):
 
 
 def epanechnikov_kernel(t, T, bandwidth=1.):
-    M = 0.75 * (1 - (t - T) / bandwidth) ** 2
+    M = 0.75 * (1 - ((t - T) / bandwidth) ** 2)
     M[abs((t - T)) >= bandwidth] = 0
     return M
 
